/*------------------------------------------------------------------------------
   EPA - Region II, ESD
  ------------------------------------------------------------------------------
   Program: weightedjenks .c
   Purpose: calcuate jenks optimal taking into account value of real weights
  ------------------------------------------------------------------------------
   Usage:   see char *usage
  ------------------------------------------------------------------------------
   Calls:   jenkslib.c
   Calling: jenks.aml
  ------------------------------------------------------------------------------
   Notes:   
  ------------------------------------------------------------------------------
   History:
     Author/Site:    Date:       Event:
   Alan Brenner    21 Jul 93   Original coding
                   11 Jan 94   update
		   16 Aug 95   changed functions calls to jenkslib routines
     alan_brenner@unixmail.edi.epa.gov
  ------------------------------------------------------------------------------
   Disclaimer:
     Although this program has been used by the U. S. Environmental Protection
   Agency, no warranty, expressed or implied, is made by the EPA as to the
   accurarcy and functioning of the program and related program material nor
   shall the fact of distribution constitute any such warranty, and no
   responsibility is assumed by the EPA in connection herewith.
==============================================================================*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "jenkslib.h"
#include "utils.h"
#include "inforeselect.h"

char    debug=0;                                            /* debugger */

/* main program function =======================================================================================================*/
int main(argc, argv)
  int argc;
  char *argv[];
{
     char *usage="Usage: jenks <data_table> <lookup_table> <data_item> <weight_item> <classes> {polygons}";

     long classes;					    /* number of classes to put data into */

     long count;					    /* for loop counter */
     long counter;					    /* for loop counter nested in count */

     long rows;						    /* number of data items */
     long *breakp;					    /* array of the current test breakpoints */
     long *bestb;					    /* array of best breakpoint found */

     double sdam;					    /* sum of the deviations of the whole array */
     double gvf=0.0;					    /* value of the goodness of variance fit for the best breaks */

     double *points;					    /* input data values */
     unsigned long *hits;				    /* array of the number of occurance of each value in points */
     double *weights;					    /* array of the floating point weights of each value in points*/

     InfoFile *infofile;				    /* input data file */
     char *ctmp="";					    /* dummy character item for getting numbers from INFO table */
     long offset=0;					    /* record number offset */
     InfoItemDef *dataitem;				    /* data item in infofile */
     short iwidth;					    /* internal width of dataitem */
     short ioutput;					    /* display width of dataitem */
     short idecimals;					    /* number of decimal place in dataitem */
     long itype;					    /* storage type of dataitem */
     InfoItemDef *countitem;				    /* input weight item (also used in output) */
     long wtype;					    /* storage type of weights */
     unsigned long InfoReselect=0;                          /* number of reselected items--default is 0 */
     InfoReselectStruct *InfoReselects=NULL;                /* pointer to empty reselect list */

     InfoItemDef *symitem;				    /* output table items */
     unsigned long total;				    /* number of data values in each class */
     double next;					    /* output temp values */
     double last;


/* command line switches -------------------------------------------------------------------------------------------------------*/
     if (strncmp(argv[0]+strlen(argv[0])-10,"inforegexd",10)==0)
          debug=9;                                          /* set debug output to max if called as inforegexd */

     if (debug > 8)                                         /* print command line parameters */
     {
          BUGOUT(stderr,"\nSTART\n");
          for (count=0; count<argc; count++)
               BUGOUT(stderr,"arg %d=%s\n",count,argv[count]);
     }

     NEXTARG;
     while ((argc>0) && (argv[0][0]=='-'))                  /* if there are arguements and the first character is a - */
     {
          switch (argv[0][1])
          {
             case 'd':                                      /* set the debug value */
               if (strlen(argv[0])>2)                       /* if there is something after the d */
               {
                    if (isdigit(argv[0][2])!=0)             /* if it is a digit */
                         debug=(char) atoi(&argv[0][2]);    /* set the debug value to it */
                    else
                         debug=1;                           /* otherwise minimal debuging */
		    NEXTARG;
               }
               else                                         /* chack the next agruement */
               {
                    if (argc>1)                             /* if there is another argument, */
                    {
			 NEXTARG;
                         if (isdigit(argv[0][0])!=0)        /* is it a number? */
                         {
                              debug=(char) atoi(argv[2]);   /* if yes, set the debug level to it */
			      NEXTARG;
                         }
			 else
			      debug=1;			    /* not a digit, set minimal debug */
                    } /* If there are no more arguments, then the usage check will be true and exit */
               }
               break;

             case 'r':
               if (strlen(argv[0])>2)                       /* if there is something after the r */
	       {
                    InfoReselect=InfoGetReselects(&(argv[0][2]),&InfoReselects);
		    NEXTARG;
	       }
               else                                         /* check the next agruement */
               {
                    if (argc>1)                             /* is there another argument? */
                    {
			 NEXTARG;
                         if (isdigit(argv[0][0])!=0)        /* is it a number? */
                         {
			      InfoReselect=InfoGetReselects(argv[1],&InfoReselects);
			      NEXTARG;
			 }
                    }
               }
               break;

             default:                                       /* check usage */
               error(1,usage);
          }
     }

     if (debug > 8)                                         /* print remaining command line arguments */
     {
          for (count=0; count<argc; count++)
               BUGOUT(stderr,"arg %d=%s\n",count,argv[count]);
          BUGOUT(stderr,"check for file\n");
     }
     if (argc<6) error(1,usage);                            /* not enough arguments after switches */

/* get the data ----------------------------------------------------------------------------------------------------------------*/
     classes=atoi(argv[4]);				    /*gets number of classes from command line*/

     if (classes<2)					    /*checks for screwy input*/
	  error(1,"The number of classes must be greater than one.");

     if ((infofile=InfoOpenFile(argv[1],InfoREAD))==(InfoFile*)NULL)
	  error(2,"Could no open input info file.");	    /* open data file */
     rows=InfoNumberRecords(infofile);			    /* get number of data records */

     if ((dataitem=InfoGetItemDef(infofile,argv[3]))==(InfoItemDef*)NULL)
          error(3,"Could not open data item.");             /* find data item */
     iwidth=InfoItemWidth(dataitem);
     ioutput=InfoItemOutputWidth(dataitem);
     itype=InfoItemType(dataitem);
     idecimals=InfoItemNumberDecimals(dataitem);

     if ((countitem=InfoGetItemDef(infofile,argv[4]))==(InfoItemDef*)NULL)
	  error(3,"Could not open weight item.");	    /* get count item */
     wtype=InfoItemType(countitem);

     if (argc>6)
     {
	  offset=2;
	  rows--;					    /*if polygon, decrease rows by one*/
     }
     else
	  offset=1;

     points=calloc(rows,sizeof(double));		    /*get data space*/
     if (wtype==INFO_BINARY_TYPE || wtype==INFO_INTEGER_TYPE)
     {
	  weights=malloc(sizeof(double));		    /* one value for input */
	  hits=calloc(rows,sizeof(long));		    /* array of counts */
     }
     else
     {
	  weights=calloc(rows,sizeof(double));		    /* array of weights */
	  hits=NULL;					    /* not needed */
     }

     for (count=0; count<rows; count++)			    /* read data */
     {
	  if (InfoReadRecord(infofile,count+offset)==INFO_FAIL)
	       error(4,"Error reading info file.");	    /* get record */

	  if (InfoDecode(infofile,dataitem,ctmp,points+count)==INFO_FAIL)
	       error(4,"Error decoding data item.");	    /* extract data item */

	  if (wtype==INFO_BINARY_TYPE || wtype==INFO_INTEGER_TYPE)
	  {
	       if (InfoDecode(infofile,countitem,ctmp,weights)==INFO_FAIL)
		    error(4,"Error decoding weight item."); /* extract count item */
	       hits[count]=(long) weights;
	  }
	  else
	  {
	       if (InfoDecode(infofile,countitem,ctmp,weights+count)==INFO_FAIL)
		    error(4,"Error decoding weight item."); /* extract weight item */
	  }
     }
     if (InfoCloseFile(infofile)==INFO_FAIL)
          error(4,"Error closing info file.");

     makelut(argv[2],argv[3],iwidth,ioutput,itype,idecimals); /* create lookup table--done here to avoid doing the work */
							    /* and then find an INFO limitation for the output (999 tables) */

     if (wtype==INFO_BINARY_TYPE || wtype==INFO_INTEGER_TYPE)
     {
	  ssortdul(rows,points,hits);			    /* sort data of integer weights */
	  free(weights);
	  weights=NULL;
     }
     else
	  ssortdd(rows,points,weights);			    /* sort data of double weights */

     if (classes-1>=rows)				    /* checks for screwy input */
	  error(5,"The number of classes must be smaller than the number of different data points.");

/* initialize break point arrays and find the best breaks ----------------------------------------------------------------------*/
     breakp=calloc(((classes+1)*classes),sizeof(long));	    /*memory for pointers to breakpoints*/
     bestb=calloc(classes+1,sizeof(long));		    /*memory for best breakpoints*/

     breakp[0]=bestb[0]=-1;				    /*beginning of list*/
     breakp[classes]=bestb[classes]=rows-1;		    /*end of raw list*/

     for (count=1; count<classes; count++)		    /*set initial breakpoints*/
	  bestb[count]=breakp[count]=count-1;		    /*point to data value before break*/

     sdam=ssd(rows,points,hits,weights);		    /* get deviation of entire array */
     if (sdam==0)
	  error(2,"Zero variability--Cannot calculate breakpoints.");

     gvf=step(classes,classes-1,gvf,sdam,bestb,breakp,points,hits,weights,0,NULL,0); /* find best gvf */

     (void) printf("\n\n\nThe Goodness of Variance Fit for %d classes = %f.\n\n",classes,gvf);

/* output the results ----------------------------------------------------------------------------------------------------------*/
     openlut(&infofile,argv[2],&dataitem,argv[3],&symitem,&countitem); /* open lookup table */

     last=points[0]-1;					    /*first line is one smaller than smallest*/
     writelut(infofile,dataitem,last,symitem,(double)0,countitem,(double)rows,1);

     for (count=1; count<classes; count++)		    /* for each breakpoint */
     {
	  next=(points[bestb[count]]+points[bestb[count]+1])/2; /* breaks are mean of value pointed at & following */

	  for (total=counter=0; counter<rows; counter++)    /*find number of values in class*/
	  {
	       if (wtype==INFO_BINARY_TYPE || wtype==INFO_INTEGER_TYPE)
	       {
		    if (points[counter]>last && points[counter]<next) total+=hits[counter];
	       }
	       else
	       {
		    if (points[counter]>last && points[counter]<next) total++;
	       }
	  }
	  writelut(infofile,dataitem,next,symitem,(double)count,countitem,(double)total,count+1);
	  last=next;
     }

     for (total=counter=0; counter<rows; counter++)	    /*find number of values in last class*/
     {
	  if (wtype==INFO_BINARY_TYPE || wtype==INFO_INTEGER_TYPE)
	  {
	       if (points[counter]>last && points[counter]<points[rows-1]+1) total+=hits[counter];
	  }
	  else
	  {
	       if (points[counter]>last && points[counter]<points[rows-1]+1) total++;
	  }
     }
     writelut(infofile,dataitem,points[rows-1]+1,symitem,(double)rows,countitem,(double)total,classes+1);

     if (InfoFileFlush(infofile)==INFO_FAIL)                /* flush and close info file */
          error(14,"Error flushing info file.");
     if (InfoCloseFile(infofile)==INFO_FAIL)
          error(14,"Error closing info file.");

     return(0);						    /*return no errors*/
}
