/*statistical functions some simple and some (many) derived from: */
/* William H. Press, Brian P. Flannery, Saul A. Teukolsky and William T. Vettering */
/* NUMERICAL RECIPIES in C: The Art of Scientific Computing */
/* Cambridge University Press, 1988 */
/*written by Alan Brenner 16 November 1992 and reworked 12 Jan 94*/

#include "stats.h"

#define TINY 1.0e-30

double cta(nn, ni, nj, chisq, df, cramrv, ccc)
  long *nn[],ni,nj;
  double *chisq,*df,*cramrv,*ccc;
{
     long nnj,nni,j,i,minij;
     double sum=0.0,expctd,*sumi,*sumj,t;

     if ((sumi=calloc((unsigned int)ni,sizeof(double)))==NULL)
	  error(-1,"Unable to allocate memory in function cta.");
     if ((sumj=calloc((unsigned int)nj,sizeof(double)))==NULL)
	  error(-2,"Unable to allocate memory in function cta.");
     nni=ni;
     nnj=nj;

     for (i=0; i<ni; i++)
     {
	  sumi[i]=0.0;
	  for (j=0; j<nj; j++)
	  {
	       sumi[i]+=nn[i][j];
	       sum+=nn[i][j];
	  }
	  if (sumi[i]==0.0) --nni;
     }
     for (j=0; j<nj; j++)
     {
	  sumj[j]=0.0;
	  for (i=0; i<ni; i++)
	       sumj[j]+=nn[i][j];
	  if (sumj[j]==0.0) --nj;
     }

     *df=nni*nnj-nni-nnj+1;
     *chisq=0.0;
     for (i=0; i<ni; i++)
     {
	  for (j=0; j<nj; j++)
	  {
	       expctd=sumj[j]*sumi[i]/sum;
	       t=nn[i][j]-expctd;
	       *chisq+=t*t/(expctd+TINY);
	  }
     }
     minij=nni<nnj ? nni-1 : nnj-1;
     *cramrv=sqrt(*chisq/(sum*minij));
     *ccc=sqrt(*chisq/(*chisq+sum));
     free(sumi);
     free(sumj);
     return(gammq(*df/2.0,*chisq/2.0));
}
