/*statistical functions some simple and some (many) derived from: */
/* William H. Press, Brian P. Flannery, Saul A. Teukolsky and William T. Vettering */
/* NUMERICAL RECIPIES in C: The Art of Scientific Computing */
/* Cambridge University Press, 1988 */
/*written by Alan Brenner 16 November 1992 and reworked 12 Jan 94*/

#include "stats.h"

#define TINY 1.0e-30

double ecta(nn, ni, nj, h, hx, hy, hygx, hxgy, uygx, uxgy, uxy)
  long *nn[],ni,nj;
  double *h,*hx,*hy,*hygx,*hxgy,*uygx,*uxgy,*uxy;
{
     long i,j;
     double sum=0.0,p,*sumi,*sumj;

     if ((sumi=calloc((unsigned int) ni,sizeof(double)))==NULL)
	  error(-1,"Unable to allocate memory in ecta.");
     if ((sumj=calloc((unsigned int) nj,sizeof(double)))==NULL)
	  error(-2,"Unable to allocate memory in ecta.");

     for (i=0; i<ni; i++)
     {
          sumi[i]=0.0;
          for (j=0; j<nj; j++)
          {
               sumi[i]+=nn[i][j];
               sum+=nn[i][j];
          }
          if (sumi[i]==0.0) --ni;
     }
     for (j=0; j<nj; j++)
     {
          sumj[j]=0.0;
          for (i=0; i<ni; i++)
               sumj[j]+=nn[i][j];
          if (sumj[j]==0.0) --nj;
     }

     *hx=0.0;
     for (i=0; i<ni; i++)
	  if (sumi[i])
	  {
	       p=sumi[i]/sum;
	       *hx-=p*log(p);
	  }
     *hy=0.0;
     for (j=0; j<nj; j++)
	  if (sumj[j])
	  {
	       p=sumj[j]/sum;
	       *hy-=p*log(p);
	  }
     *h=0.0;
     for (i=0; i<ni; i++)
     {
	  for (j=0; j<nj; j++)
	  {
	       if (nn[i][j])
	       {
		    p=nn[i][j]/sum;
		    *h-=p*log(p);
	       }
	  }
     }
     *hygx=*h-*hx;
     *hxgy=*h-*hy;
     *uygx=(*hy-*hygx)/(*hy+TINY);
     *uxgy=(*hx-*hxgy)/(*hx+TINY);
     *uxy=2.0*(*hy+*hy-*h)/(*hx+*hy+TINY);
     free(sumi);
     free(sumj);
     return(*uxy);
}
