/*statistical functions some simple and some (many) derived from: */
/* William H. Press, Brian P. Flannery, Saul A. Teukolsky and William T. Vettering */
/* NUMERICAL RECIPIES in C: The Art of Scientific Computing */
/* Cambridge University Press, 1988 */
/*written by Alan Brenner 16 November 1992 and reworked 12 Jan 94*/

#include "stats.h"

#define BIG 1.0e30
#define AFAC 1.5
#define AMP 1.5

double median(n, data)					    /*calculate median*/
  long n;
  double *data;
{							    /* from Press et al, p478 */
     int np,nm,j;
     double xx,xp,xm,sumx,sum,eps,stemp,dum,ap,am,aa,a=(data[0]+data[n])/2;

     eps=fabs(data[n]-data[0]);
     am=-(ap=BIG);
     for (;;)
     {
	  sum=sumx=0.0;
	  np=nm=0;
	  xm=-(xp=BIG);

	  for (j=0;j<n;j++)
	  {
	       xx=data[j];
	       if (xx!=a)
	       {
		    if (xx>a)
		    {
			 ++np;
			 if (xx<xp) xp=xx;
		    }
		    else if (xx<a)
		    {
			 ++nm;
			 if (xx>xm) xm=xx;
		    }
		    sum+=dum=1.0/(eps+fabs(xx-a));
		    sumx+=xx*dum;
	       }
	  }
	  stemp=sumx/sum-a;
	  if (np-nm>-2)
	  {
	       am=a;
	       aa=stemp<0.0 ? xp : xp+stemp*AMP;
	       if (aa>ap) aa=(a+ap)/2.0;
	       eps=AFAC*fabs(aa-a);
	       a=aa;
	  }
	  else if (nm-np>=2)
	  {
	       ap=a;
	       aa=stemp>0.0 ? xm : xm+stemp*AMP;
	       if (aa<am) aa=(a+am)/2.0;
	       eps=AFAC*fabs(aa-a);
	       a=aa;
	  }
	  else
	  {
	       if (n%2==0)
		    return((np==nm ? xp+xm : np>nm ? a+xp : xm+a)/2);
	       else
		    return(np==nm ? a : np > nm ? xp : xm);
	  }
     }
}
