/*---------------------------------------------------------------------------------------------------------------------------------
   EPA - Region II, ESD
  ------------------------------------------------------------------------------
   Program: chisquare1()
   Purpose: chi squared test of data vs expected
  ------------------------------------------------------------------------------
   Usage:   see function
  ------------------------------------------------------------------------------
   Calls:   gammq()
   Calling: general function
  ------------------------------------------------------------------------------
   Notes:   statistical functions some simple and some derived from:
William H. Press, Brian P. Flannery, Saul A. Teukolsky and William T. Vettering
    _NUMERICAL RECIPIES in C: The Art of Scientific Computing_
    Cambridge University Press, 1988
            This book is useful, but contains problems--most notably incorrect
	    subscripting of arrays (C is 0 to N-1; they use 1 to N).
  ------------------------------------------------------------------------------
   History:
     Author/Site:    Date:       Event:
   Alan Brenner      Jan 94      original coding
     alan_brenner@unixmail.edi.epa.gov
   Alan Brenner      Jan 95      added case based statistics
                                 added debuging
  ------------------------------------------------------------------------------
   Disclaimer:
     Although this program has been used by the U. S. Environmental Protection
   Agency, no warranty, expressed or implied, is made by the EPA as to the
   accurarcy and functioning of the program and related program material nor
   shall the fact of distribution constitute any such warranty, and no
   responsibility is assumed by the EPA in connection herewith.
---------------------------------------------------------------------------------------------------------------------------------*/

#include "stats.h"

double chisquare1(rows, data, expect, constrn, df, prob)
  long rows;						    /* number of items in data[] */
  long constrn;						    /* external constraints on degrees of freedom */
  double *data;						    /* input data */
  double *expect;					    /* theoretically expected value */
  double *df;						    /* output degrees of freedom */
  double *prob;						    /* output probability of chi squared value */
{
     int count;						    /* for loop counter */
     double t;						    /* temporary value */
     double chisq=0.0;					    /* output chi square */

     if (debug) (void) fprintf(stderr,"chisquare1(%d,%p,%p,%d,%p,%p)\n",rows,data,expect,constrn,df,prob);

     *df=(double)rows-1.0-constrn;
     for (count=0; count<rows; count++)
     {
	  if (expect[count] <= 0.0)
	  {
	       (void) fprintf(stderr,"\nExpected number must be greater than zero.  Returning 0 as NODATA.\n");
	       *prob=0.0;
	       return(*prob);
	  }
	  t=data[count]-expect[count];
	  chisq+=t*t/expect[count];
     }
     *prob=gammq(*df/2.0,chisq/2.0);
     return(chisq);
}
