/*---------------------------------------------------------------------------------------------------------------------------------
   EPA - Region II, ESD
  ------------------------------------------------------------------------------
   Program: fit1()
   Purpose: find the linear regresssion coefficients for two data sets
  ------------------------------------------------------------------------------
   Usage:   see function
  ------------------------------------------------------------------------------
   Calls:   nothing
   Calling: general function
  ------------------------------------------------------------------------------
   Notes:   statistical functions some simple and some derived from:
William H. Press, Brian P. Flannery, Saul A. Teukolsky and William T. Vettering
    _NUMERICAL RECIPIES in C: The Art of Scientific Computing_
    Cambridge University Press, 1988
            This book is useful, but contains problems--most notably incorrect
	    subscripting of arrays (C is 0 to N-1; they use 1 to N).
  ------------------------------------------------------------------------------
   History:
     Author/Site:    Date:       Event:
   Alan Brenner      Jan 94      original coding
     alan_brenner@unixmail.edi.epa.gov
   Alan Brenner      Jan 95      added case based statistics
                                 added debuging
  ------------------------------------------------------------------------------
   Disclaimer:
     Although this program has been used by the U. S. Environmental Protection
   Agency, no warranty, expressed or implied, is made by the EPA as to the
   accurarcy and functioning of the program and related program material nor
   shall the fact of distribution constitute any such warranty, and no
   responsibility is assumed by the EPA in connection herewith.
---------------------------------------------------------------------------------------------------------------------------------*/

#include "stats.h"

void fit1(rows, xdata, ydata, sig, a, b, siga, sigb, chi2, q)
  long rows;						    /* number of data items in xdata[] and ydata[] */
  double *xdata;					    /* the x coordinate of the actual data pairs */
  double *ydata;					    /* the y coordinate of the actual data pairs */
  double *sig;						    /* standard deviation weights of x,y */
  double *a;						    /* y=ax+b */
  double *b;						    /* y=ax+b */
  double *siga;						    /* probable uncertainty of a */
  double *sigb;						    /* probable uncertainty of b */
  double *chi2;						    /* chi squared value */
  double *q;						    /* goodness-of-fit probability of chi2 */
{
     long count;					    /* for loop counter */
     double temp;					    /* temporary double */
     double st2=0.0;					    /* sum of temp squared */
     double sxoss;					    /* sx/ss */
     double sx=0.0;					    /* sum of (weighted) xdata[] values */
     double sy=0.0;					    /* sum of (weighted) ydata[] values */
     double ss;						    /* sum of weights or rows */

     if (debug) (void) fprintf(stderr,"fit1(%d,%p,%p,%p,%p,%p,%p,%p,%p,%p)\n",rows,xdata,ydata,sig,a,b,siga,sigb,chi2,q);

     if (sig!=(double *)NULL)				    /* sig is not a NULL pointer */
     {
	  ss=0.0;
	  for (count=0; count<rows; count++)		    /* calculate weighted summations of x and y values */
	  {
	       temp=1.0/(sig[count]*sig[count]);	    /* weight is inverse of the square of the weight value */
	       ss+=temp;				    /* summation of weight */
	       sx+=xdata[count]*temp;			    /* summation of weighted x */
	       sy+=ydata[count]*temp;			    /* summation of weighted y */
	  }
     }
     else
     {
	  for (count=0; count<rows; count++)		    /* calculate unweighted summations of x and y values */
	  {
	       sx+=xdata[count];
	       sy+=ydata[count];
	  }
	  ss=(double) rows;				    /* summation of weights of 1 == number of data items */
     }
     sxoss=sx/ss;

     *b=0.0;
     if (sig!=(double *)NULL)
     {
	  for (count=0; count<rows; count++)
	  {
	       temp=(xdata[count]-sxoss)/sig[count];
	       st2+=temp*temp;
	       *b+=temp*ydata[count]/sig[count];
	  }
     }
     else
     {
	  for (count=0; count<rows; count++)
	  {
	       temp=xdata[count]-sxoss;
	       st2+=temp*temp;
	       *b+=temp*y[count];
	  }
     }

     *b/=st2;
     *a=(sy-sx*(*b))/ss;
     *siga=sqrt((1.0+sx*sx/(ss*st2))/ss);
     sigb=sqrt(1.0/st2);
     *chi2=0.0;
     if (sig!=(double *)NULL)
     {
	  for (count=0; count<rows; count++)
	  {
	       temp=(ydata[count]-(*a)-(*b)*xdata[count])/sig[count];
	       *chi2+=temp*temp;
	  }
	  *q=gammq(0.5*(rows=2),0.5*(*chi2));
     }
     else
     {
	  for (count=0; count<rows; count++)
	  {
	       temp=(ydata[count)-(*a)-(*b)*xdata[count]);
	  }
	  *chi2+=temp*temp;
	  *q=1.0;
	  temp=(sqrt((*chi2)/rows-2));
	  *siga*=temp;
	  *sigb*=temp;
     }
}
