/*---------------------------------------------------------------------------------------------------------------------------------
   EPA - Region II, ESD
  ------------------------------------------------------------------------------
   Program: kurt()
   Purpose: find the kurtosis (peakedness) of the current data set vs bell curve
  ------------------------------------------------------------------------------
   Usage:   see function
  ------------------------------------------------------------------------------
   Calls:   nothing
   Calling: general function
  ------------------------------------------------------------------------------
   Notes:   statistical functions some simple and some derived from:
William H. Press, Brian P. Flannery, Saul A. Teukolsky and William T. Vettering
    _NUMERICAL RECIPIES in C: The Art of Scientific Computing_
    Cambridge University Press, 1988
            This book is useful, but contains problems--most notably incorrect
	    subscripting of arrays (C is 0 to N-1; they use 1 to N).
  ------------------------------------------------------------------------------
   History:
     Author/Site:    Date:       Event:
   Alan Brenner      Jan 94      original coding
     alan_brenner@unixmail.edi.epa.gov
   Alan Brenner      Jan 95      added case based statistics
                                 added debuging
  ------------------------------------------------------------------------------
   Disclaimer:
     Although this program has been used by the U. S. Environmental Protection
   Agency, no warranty, expressed or implied, is made by the EPA as to the
   accurarcy and functioning of the program and related program material nor
   shall the fact of distribution constitute any such warranty, and no
   responsibility is assumed by the EPA in connection herewith.
---------------------------------------------------------------------------------------------------------------------------------*/

#include "stats.h"

double kurt(rows, ave, var, data)
  long rows;						    /* number of items in data[] */
  double ave;						    /* the arithmatic mean of the values in data[] */
  double var;						    /* the variance of the values in data[] */
  double *data;						    /* the actual data */
{
     long count;					    /* for loop counter */
     double output=0.0;					    /* output kurtosis */

     if (debug) (void) fprintf(stderr,"kurt(%d,%f,%f,%p)\n",rows,ave,var,data);

     if (rows<2)
     {
	  (void) fprintf(stderr,"There must be more than one data value to calculate kurtosis.\n");
	  (void) fprintf(stderr,"Returning 0 as NODATA.\n");
	  return(0.0);
     }
     if (var<0)
     {
	  (void) fprintf(stderr,"Garbage input for variance--it is less than zero.\n");
	  (void) fprintf(stderr,"Returning 0 as NODATA.\n");
	  return(0.0);
     }
     if (var==0)
     {
	  (void) fprintf(stderr,"Variance must be greater than 0 to calculate kurtosis.\n");
	  (void) fprintf(stderr,"Returning 0 as NODATA.\n");
	  return(0.0);
     }

     for (count=0; count<rows; count++)			    /* for every data item */
	  output+=pow(data[count]-ave,4);		    /* summation of (the differences from the mean) to the 4th power */

     return(output/(rows*var)-3.0);			    /* return kurtosis */
}
