/*---------------------------------------------------------------------------------------------------------------------------------
   EPA - Region II, ESD
  ------------------------------------------------------------------------------
   Program: maxr()
   Purpose: find maximum data value for each case and record its record number
  ------------------------------------------------------------------------------
   Usage:   see function
  ------------------------------------------------------------------------------
   Calls:   nothing
   Calling: general function
  ------------------------------------------------------------------------------
   Notes:   statistical functions some simple and some derived from:
William H. Press, Brian P. Flannery, Saul A. Teukolsky and William T. Vettering
    _NUMERICAL RECIPIES in C: The Art of Scientific Computing_
    Cambridge University Press, 1988
            This book is useful, but contains problems--most notably incorrect
	    subscripting of arrays (C is 0 to N-1; they use 1 to N).
  ------------------------------------------------------------------------------
   History:
     Author/Site:    Date:       Event:
   Alan Brenner      Jan 94      original coding
     alan_brenner@unixmail.edi.epa.gov
   Alan Brenner      Jan 95      added case based statistics
                                 added debuging
  ------------------------------------------------------------------------------
   Disclaimer:
     Although this program has been used by the U. S. Environmental Protection
   Agency, no warranty, expressed or implied, is made by the EPA as to the
   accurarcy and functioning of the program and related program material nor
   shall the fact of distribution constitute any such warranty, and no
   responsibility is assumed by the EPA in connection herewith.
---------------------------------------------------------------------------------------------------------------------------------*/

#include "stats.h"

double maxr(rows, data, recs)
  long rows;						    /* number of data items in data[] */
  long *recs;						    /* address of a long for record number of maximums */
  double *data;						    /* the actual data */
{
     long count;					    /* for loop counter */
     double output;					    /* the maximum data value */

     if (debug) (void) fprintf(stderr,"maxr(%d,%p,%p)\n",rows,data,recs);

     output=data[0];					    /* initialize output with something valid for comparison */
     *recs=1;						    /* point to the first item */

     for (count=1; count<rows; count++)			    /* for every other data item */
     {
	  if (output<data[count])			    /* if the current maximum value is smaller than this data value */
	  {
	       output=data[count];			    /* save the larger value */
	       *recs=count+1;				    /* and the record number that it occurs on */
	  }
     }
     return(output);					    /* send the maximum data value back as the function's return */
}
