/*---------------------------------------------------------------------------------------------------------------------------------
   EPA - Region II, ESD
  ------------------------------------------------------------------------------
   Program: median()
   Purpose: find median data value
  ------------------------------------------------------------------------------
   Usage:   see function
  ------------------------------------------------------------------------------
   Calls:   nothing
   Calling: general function
  ------------------------------------------------------------------------------
   Notes:   statistical functions some simple and some derived from:
William H. Press, Brian P. Flannery, Saul A. Teukolsky and William T. Vettering
    _NUMERICAL RECIPIES in C: The Art of Scientific Computing_
    Cambridge University Press, 1988
            This book is useful, but contains problems--most notably incorrect
	    subscripting of arrays (C is 0 to N-1; they use 1 to N).
  ------------------------------------------------------------------------------
   History:
     Author/Site:    Date:       Event:
   Alan Brenner      Jan 94      original coding
     alan_brenner@unixmail.edi.epa.gov
   Alan Brenner      Jan 95      added case based statistics
                                 added debuging
  ------------------------------------------------------------------------------
   Disclaimer:
     Although this program has been used by the U. S. Environmental Protection
   Agency, no warranty, expressed or implied, is made by the EPA as to the
   accurarcy and functioning of the program and related program material nor
   shall the fact of distribution constitute any such warranty, and no
   responsibility is assumed by the EPA in connection herewith.
---------------------------------------------------------------------------------------------------------------------------------*/

#include "stats.h"

double median(rows, data)
  long rows;						    /* number of items in data[] */
  double *data;						    /* array of data items */
{
     int (*choose)()=dpick;				    /* pointer for qsort */
     long count;					    /* for loop counter */
     long array;					    /* array subscript */
     double *hold;					    /* local copy of data[] */

     if (debug) (void) fprintf(stderr,"median(%d,%p)\n",rows,data);

     if ((hold=calloc(rows,sizeof(double)))==NULL)	    /* get space for local copy of data[] */
	  error(2,"Cannot allocate memory in function median.");

     for (count=0; count<rows; count++)
     {
	  hold[count]=data[count];			    /* make copy of data[] */
	  if (debug>8) (void) fprintf(stderr,"%10d = %f\n",count,hold[count]);
     }

     qsort(hold,rows,sizeof(double),choose);		    /* sort temporary copy */
     if (debug>7)
     {
	  for (count=0; count<rows; count++)
	       (void) fprintf(stderr,"%10d = %f\n",count,hold[count]);
     }

     array=rows/2;					    /* half of the total number of items */
     if (rows%2==0)					    /* is rows even? */
     {
	  if (debug>6) (void) fprintf(stderr,"midpoint between %10d and %10d\n",array-1,array);
	  return((hold[array-1]+hold[array])/2.0);	    /* if yes, return mean of two middle items */
     }
     else
     {
	  if (debug>6) (void) fprintf(stderr,"midpoint is %10d\n",array);
	  return(hold[array]);				    /* if no, return middle item */
     }
}
