/*---------------------------------------------------------------------------------------------------------------------------------
   EPA - Region II, ESD
  ------------------------------------------------------------------------------
   Program: svar()
   Purpose: find the sample variance
  ------------------------------------------------------------------------------
   Usage:   see function
  ------------------------------------------------------------------------------
   Calls:   nothing
   Calling: general function
  ------------------------------------------------------------------------------
   Notes:   statistical functions some simple and some derived from:
William H. Press, Brian P. Flannery, Saul A. Teukolsky and William T. Vettering
    _NUMERICAL RECIPIES in C: The Art of Scientific Computing_
    Cambridge University Press, 1988
            This book is useful, but contains problems--most notably incorrect
	    subscripting of arrays (C is 0 to N-1; they use 1 to N).
  ------------------------------------------------------------------------------
   History:
     Author/Site:    Date:       Event:
   Alan Brenner      Jan 94      original coding
     alan_brenner@unixmail.edi.epa.gov
   Alan Brenner      Jan 95      added case based statistics
                                 added debuging
  ------------------------------------------------------------------------------
   Disclaimer:
     Although this program has been used by the U. S. Environmental Protection
   Agency, no warranty, expressed or implied, is made by the EPA as to the
   accurarcy and functioning of the program and related program material nor
   shall the fact of distribution constitute any such warranty, and no
   responsibility is assumed by the EPA in connection herewith.
---------------------------------------------------------------------------------------------------------------------------------*/

#include "stats.h"

double svar(rows, ave, data)
  long rows;						    /* number of items in data[] */
  double ave;						    /* mean of items in data[] */
  double *data;						    /* data items */
{
     long count;					    /* for loop counter */
     double var=0.0;					    /* sum of squares of differences from the mean */

     if (debug) (void) fprintf(stderr,"svar(%d,%f,%p)\n",rows,ave,data);

     for (count=0; count<rows; count++)			    /* for every data item */
	  var+=(data[count]-ave)*(data[count]-ave);	    /* calculate sum of squares of the difference from mean */
     return(var/(rows-1));				    /* return sample variance */
}
