/*---------------------------------------------------------------------------------------------------------------------------------
   EPA - Region II, ESD
  ------------------------------------------------------------------------------
   Program: ssortdl()
   Purpose: sort paired data (one double, one long) by the value in item one
  ------------------------------------------------------------------------------
   Usage:   see function
  ------------------------------------------------------------------------------
   Calls:   nothing
   Calling: general function
  ------------------------------------------------------------------------------
   Notes:   statistical functions some simple and some derived from:
William H. Press, Brian P. Flannery, Saul A. Teukolsky and William T. Vettering
    _NUMERICAL RECIPIES in C: The Art of Scientific Computing_
    Cambridge University Press, 1988
            This book is useful, but contains problems--most notably incorrect
	    subscripting of arrays (C is 0 to N-1; they use 1 to N).

	    derived from the Shell's Method routine in Press, p243
------------------------------------------------------------------------------
   History:
     Author/Site:    Date:       Event:
   Alan Brenner      Jan 94      original coding
     alan_brenner@unixmail.edi.epa.gov
   Alan Brenner      Jan 95      added case based statistics
                                 added debuging
  ------------------------------------------------------------------------------
   Disclaimer:
     Although this program has been used by the U. S. Environmental Protection
   Agency, no warranty, expressed or implied, is made by the EPA as to the
   accurarcy and functioning of the program and related program material nor
   shall the fact of distribution constitute any such warranty, and no
   responsibility is assumed by the EPA in connection herewith.
---------------------------------------------------------------------------------------------------------------------------------*/

#include "utils.h"
#define ALN2I 1.442695022
#define TINY 1.0E-5

void ssortdul(rows, data1, data2)
  long rows;						    /* number of data items */
  double *data1;					    /* double data item (sort item) */
  unsigned long *data2;					    /* unsigned long data item */
{
     if (rows<50)					    /* use straight insertion for small data sets */
     {
	  long i,j;					    /* for loop counters */
	  unsigned long t2;				    /* temporary storage for long */
	  double t1;					    /* temporary storage for double*/

	  if (debug>7) (void) fprintf(stderr,"\nsmall data set--ssortpd %d\n",rows);
	  if (debug>8)					    /* print input data */
	  {
	       long i;
	       (void) fprintf(stderr,"UNSORTED:\n");
	       for (i=0; i<rows; i++) fprintf(stderr,"%d\t%f\t\t%f\n",i,data1[i],data2[i]);
	  }

	  for (j=1; j<rows; j++)			    /* for every data item after first */
	  {
	       t1=data1[j];				    /* make a temporary copy */
	       t2=data2[j];

	       i=j-1;					    /* this starts at 0--for at most one iteration of this loop */
	       while (i>=0 && data1[i]>t1)		    /* for each data item back to the beginning */
	       {
		    data1[i+1]=data1[i];		    /* move the data back on spot in the array */
		    data2[i+1]=data2[i];
		    i--;				    /* decrement the current point in the array */
	       }
	       data1[i+1]=t1;				    /* put the temporary data in the hole left by the while loop */
	       data2[i+1]=t2;
	  }

	  if (debug>7)					    /* print sorted data */
	  {
	       long i;
	       (void) fprintf(stderr,"SORTED:\n");
	       for (i=0; i<rows; i++) fprintf(stderr,"%d\t%f\t\t%f\n",i,data1[i],data2[i]);
	  }
     }
     else
     {
	  long i,j,k,l;					    /* for loop counters */
	  unsigned long t2;				    /* temporary storage for unsigned long */
	  double t1;					    /* temporary storage for double*/
	  double lognb2;				    /* holder for number of iterations */

	  if (debug>7) (void) fprintf(stderr,"\nlarge data set--ssortpd %d\n",rows);
	  if (debug>8)					    /* print input data */
	  {
	       long i;
	       (void) fprintf(stderr,"UNSORTED:\n");
	       for (i=0; i<rows; i++) fprintf(stderr,"%d\t%f\t\t%f\n",i,data1[i],data2[i]);
	  }

	  t1=(double) rows;
	  t2=log(t1);
	  lognb2=t2*ALN2I+TINY;				    /* determine number of total iterations */
	  k=rows;
	  if (debug>7) (void) fprintf(stderr,"k=%d\t\tt1=%f\t\tt2=%f\t\tlognb2=%f\n",k,t1,t2,lognb2);

	  for (l=0; l<lognb2; l++)			    /* for each subset */
	  {
	       k>>=1;
	       if (debug>8) (void) fprintf(stderr,"l=%d\t\tk=%d\n",l,k);
	       
	       for (j=k; j<rows; j++)			    /* this is the above small sort with k instead of 1*/
	       {
		    t1=data1[j];
		    t2=data2[j];
		    i=j-k;
		    while (i>=0 && data1[i]>t1)
		    {
			 data1[i+k]=data1[i];
			 data2[i+k]=data2[i];
			 i-=k;
		    }
		    data1[i+k]=t1;
		    data2[i+k]=t2;
	       }
	  }
	  
	  if (debug>7)					    /* print sorted data */
	  {
	       long i;
	       (void) fprintf(stderr,"SORTED:\n");
	       for (i=0; i<rows; i++) fprintf(stderr,"%d\t%f\t\t%f\n",i,data1[i],data2[i]);
	  }
     }
}
